import React, { useEffect, useLayoutEffect, memo, useState } from 'react'
import BaseTmp from '@/components/BaseTmp'
import { Row, Select, Button, Table, Col, Tabs, Divider, Popconfirm, Modal, Input, message } from 'antd'
import api from '@/services/api/index'
import { PlusOutlined } from '@ant-design/icons'
import { descText } from '@/common/MessageManage' 
const { getQuestionBank, getQuestionBook, postQuestionBook, putQuestionBook } = api.question

const { TabPane } = Tabs

interface IBank {
  id: string | number,
  label: string
}

interface IQueryTmp {
  bankList: IBank[],
  onChangeBank: ( id: string ) => void,
  openModel: (type: 'add') => void,
  currentBank: IBank
}

const QueryTmp: React.FC<IQueryTmp> = ({
  bankList,
  onChangeBank,
  openModel,
  currentBank
}) => {

  const [ bank, setBank ] = useState<IBank>()

  useEffect( () => {
    setBank(bankList[0])
  }, [bankList])
  
  return <Row justify="space-between" >
    <Col span={20} >

      <Tabs onChange={onChangeBank} >
        {
          bankList.map( bank =>  <TabPane tab={bank.label} key={bank.id} />)
        }
      </Tabs>
    </Col>
    <Col span={2} >
      <Button
        type="primary" 
        icon={<PlusOutlined />}
        onClick={() => openModel('add')}
      >
        新增教材
      </Button>
    </Col>

  </Row>
}


interface ITableFn {
  loading: boolean,
  dataSource: any[],
  onDelBook: ( record: IDataSource ) => void,
  openModel: (type: 'edit' , record: IDataSource) => void
}

const TabletTmp: React.FC<ITableFn> = ({
  loading,
  dataSource,
  onDelBook,
  openModel
}) => {

  const columns = [
    {
      title: '题卡',
      dataIndex: 'label',
      key: 'label',
    },
    {
      title: '卡片数',
      dataIndex: 'cardNum',
      key: 'cardNum',
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      key: 'createTime',
    },
    // {
    //   title: '启用',
    //   dataIndex: 'state',
    //   key: 'state',
    //   render: () => <Switch />
    // },
    {
      title: '操作',
      dataIndex: 'action',
      key: 'action',
      render: (text: any, record: IDataSource) => (
        <>
          <a  onClick={() => openModel('edit', record )} >编辑</a>
          {/* <Divider type="vertical" /> */}
          {/* <Popconfirm
            title="确定要删除这本教材吗？"
            onConfirm={() => onDelBook(record)}
            okText="是"
            cancelText="我点错了"
          >
             <a  >删除</a>
          </Popconfirm> */}
        </>
      )
    },
  ]

  return <Table style={{marginTop: "20px"}} loading={loading} dataSource={dataSource} columns={columns} />
}

interface IDataSource {
  id: string,
  label: string,
  bankId: string,
  cardNum: number,
  createTime: string
}

const QuestionBook: React.FC = () => {

  const [ bankList, setBankList ] = useState<IBank[]>([])

  const [ currentBank, setCurrentBank ] = useState<IBank>({id: '', label: ''})

  const [ loading, setLoading ] = useState<boolean>(false)

  const [ dataSource, setDataSource ] = useState<IDataSource[]>([])

  const [ currentDataSource, setCurrentDataSource ] = useState<IDataSource>({
    id: '',
    label: '',
    cardNum: 0,
    createTime: '',
    bankId: '' 
  })

  const [ confirmLoading, setConfirmLoading ] = useState<boolean>(false)
  
  const [ isModalVisible, setIsModalVisible ] = useState<boolean>(false)

  const [ type, setType ] = useState<'edit' | 'add'>('add')
  
  useEffect( () => {
    _getQuestionBank()
  }, [])

  useEffect( () => {
    console.log(currentBank, 'currentBank');
    
    currentBank.id && _getQuestionBook()
  }, [currentBank.id])

  const modelTitle = type === 'edit' ? '编辑教材' : '新增教材'

  const onChangeBank = (id: string) => {
    const targetBank = bankList.find( bank => bank.id === id )
    setCurrentBank(targetBank!)
  }

  const onInput = (e: InputEvent) => {
    setCurrentDataSource({
      ...currentDataSource!,
      label:  e?.target?.value
    })
  }

  /**删除教材 */
  const onDelBook = (record: IDataSource) => {

  }

  const handleOk = () => {

    if (type === 'add' ) {
      _postQuestionBook()
    } else {
      _putQuestionBook()
    }
  }

  /**打开弹窗 */
  const openModel = (type: 'edit' | 'add', record?: IDataSource) => {

    setType(type)
    setIsModalVisible(true)
    setConfirmLoading(false)

    if (type === 'edit') {
      setCurrentDataSource(record!)
    }

  }

  /**关闭弹窗 */
  const closeModel = () => {
    setCurrentDataSource({
      id: '',
      bankId: '',
      label: '',
      cardNum: 0,
      createTime: ''
    })
    setIsModalVisible(false)
  } 

  /**编辑教材 */
  const _putQuestionBook = async () => {
    setConfirmLoading(true)
    const {status, result} = await putQuestionBook(currentDataSource!.id, {
      label: currentDataSource!.label
    })
    closeModel()
    if (status === 200) {
      message.success(descText.editTextSuccess)
      _getQuestionBook()
    }
  }

  /**新增教材 */
  const _postQuestionBook = async () => {
    if ( currentDataSource.label.length === 0 ) {
      message.error("请输入教材名")
      return
    }
    setConfirmLoading(true)
    const { status, result } = await postQuestionBook({
      bankId: currentBank.id,
      label: currentDataSource.label
    })
    closeModel()
    if ( status === 200 ) {
      console.log(result);
      _getQuestionBook()
      message.success(descText.addTextSuccess)
    }
    
  }

  /** 查教材 */
  const _getQuestionBook = async () => {
    setLoading(true)
    const {status, result} = await getQuestionBook({
      bankId: currentBank.id
    })
    setLoading(false)
    if ( status === 200) {
      console.log(result);
      setDataSource(result)
    }
  }

  /**查题库 */
  const _getQuestionBank = async () => {
    const { status, result } = await getQuestionBank()
    if (status === 200) {
      setBankList(result)
      setCurrentBank(result[0])
    }
  } 

  return (
    <BaseTmp>

      <QueryTmp
        bankList={bankList}
        currentBank={currentBank}
        onChangeBank={onChangeBank}
        openModel={openModel}
      />

      <TabletTmp
        loading={loading}
        dataSource={dataSource}
        onDelBook={onDelBook}
        openModel={openModel}
      />

      <Modal
        title={modelTitle}
        confirmLoading={confirmLoading}
        visible={isModalVisible}
        onOk={handleOk}
        onCancel={closeModel}
      > 
        <Row>
          <Col span={4} style={{textAlign: "right"}} >学段选择：</Col>
          <Col span={20} > 
            <Select key={currentBank.id} style={{width: "150px"}} defaultValue={currentBank.id} >
              {
                bankList.map( bank =>  <Select.Option  key={bank.id} value={bank.id}  > { bank.label} </Select.Option>  )
              }
            </Select>
          </Col>
        </Row>  
        <Row  style={{marginTop: '20px'}}>
          <Col span={4}  style={{textAlign: "right"}} >教材名：</Col>
          <Col span={20} > 
            <Input
              min={1}
              placeholder='请输入教材名' 
              value={currentDataSource!.label} 
              onInput={onInput}
            />
          </Col>
        </Row>


       
       
      </Modal>
    </BaseTmp>
  ) 
}

export default QuestionBook